# -*- coding: utf-8 -*- 
# @Time    : 04/04/2024 
# @Author  : nzooherd 
# @File    : schedule_app.py 
# @Software: Visual Studio Code

import asyncio
import pexpect
from pymediainfo import MediaInfo
from datetime import datetime, timedelta
from logging import getLogger
import os
import shutil
import time

import schedule
from aspect.task import task_logger
from bootstrap import BaseApp
from config import Config
from service.resource_service import ResourceService


logger = getLogger(__name__)
class ScheduleApp(BaseApp):

    def __init__(self) -> None:
        super().__init__()

        self.file2size = {}
        self.resource_service = ResourceService()
        self.processing_file = set()

        try:
            self.loop = asyncio.get_event_loop()
        except RuntimeError:
            self.loop = asyncio.new_event_loop()
            asyncio.set_event_loop(self.loop)
   
    def register(self):
        schedule.every(10).seconds.do(self.new_video)
        schedule.every(60).seconds.do(self.carry_video)

    @task_logger
    def carry_video(self):
        if self.file2size:
            logger.info("Processing video, skip carry video")
            return

        RAW_PATH = os.path.join(Config.RESOURCE_ROOT_PATH, "raw")
        root_pikpak = "/home/nzooherd/pikpak/oneesan/JAV"
        folders = list(map(lambda x: os.path.join(root_pikpak, x), os.listdir(root_pikpak)))
        for folder in folders:
            items = os.listdir(folder)
            if "movie.nfo" not in items:
                continue
            for item in items:
                filesize = os.path.getsize(os.path.join(folder, item))
                if not item.endswith(".mp4") or filesize <= 2 * 1024 * 1024 * 1024:
                    continue
                
                _, _, free = shutil.disk_usage("/")
                if free < 1 * 1024 * 1024 * 1024 + filesize:
                    logger.warning("Disk is full")
                    return

                if item in os.listdir(RAW_PATH):
                    break

                loop = asyncio.get_event_loop()
                if loop.run_until_complete(self.resource_service.exist_video(item)):
                    break

                time.sleep(12)
                if self.file2size:
                    logger.info("Processing video, skip carry video")
                    return
                
                logger.info(f'Carry video {item} from {folder}')
                shutil.copy(os.path.join(folder, item), os.path.join(RAW_PATH, item))
                if not self.check_mp4_file(os.path.join(RAW_PATH, item)):
                    logger.error(f'The video {item} is not a mp4 file.')
                    os.remove(os.path.join(RAW_PATH, item))
                else:
                    self.restart_service("pikpak")
                logger.info(f'Carry video {item} done')
                return


    def restart_service(self, service_name):
        command = f'sudo systemctl restart {service_name}'
        child = pexpect.spawn(command)

        # 等待提示输入密码
        child.expect('.*password.*')

        # 输入密码
        child.sendline("swh123qwe")

        # 捕获命令输出和错误信息
        child.expect(pexpect.EOF)
        output = child.before.decode('utf-8')

    def check_mp4_file(self, file_path):
        media_info = MediaInfo.parse(file_path)
        for track in media_info.tracks:
            if track.track_type == "Video":
                return True
        return False

    @task_logger
    def new_video(self):
        RAW_PATH = os.path.join(Config.RESOURCE_ROOT_PATH, "raw")
        for file in os.listdir(RAW_PATH):
            file_path = os.path.join(RAW_PATH, file)
            if file_path in self.processing_file:
                continue

            if file_path not in self.file2size:
                logger.info(f'New video found: {file}')
                self.file2size[file_path] = os.path.getsize(file_path)
                continue

            if os.path.getsize(file_path) != self.file2size[file_path]:
                self.file2size[file_path] = os.path.getsize(file_path)
                continue

            logger.info(f'Video {file} is ready to be processed')
            self.processing_file.add(file_path)
            self.loop.run_until_complete(self.resource_service.new_video(file_path))
            self.processing_file.remove(file_path)
            self.file2size.pop(file_path)
        
        # 小时重试触发
        pre_hour = datetime.now() - timedelta(minutes=10)
        for folder in ["metadata", "upload", "preview"]:
            subfiles = os.listdir(os.path.join(Config.RESOURCE_ROOT_PATH, folder))
            for file in subfiles:
                file_path = os.path.join(Config.RESOURCE_ROOT_PATH, folder, file)
                if os.path.isdir(file_path):
                    continue
                if datetime.fromtimestamp(os.path.getctime(file_path)) <= pre_hour:
                    logger.event(f'Found unprocess file: {file_path}')
                    temp_file = os.path.join("/tmp", file)
                    if not os.path.exists(temp_file):
                        os.link(file_path, temp_file)
                    os.remove(file_path)
                    os.link(temp_file, file_path)
                    os.remove(temp_file)
                    time.sleep(30)


if __name__ == '__main__':
    app = ScheduleApp()
    app.register()
    while True:
        schedule.run_pending()
        time.sleep(1)